--[[
	Data Parsed from WowHead.com and tradeskillmaster.com
	Integrated Code/Concepts from Reagent Helper 2, Altoholic and others.
	Locales: English , Spanish :::  More to come
	]]--

SmexyMats = LibStub("AceAddon-3.0"):NewAddon("SmexyMats", "AceEvent-3.0", "AceConsole-3.0");

local L = LibStub("AceLocale-3.0"):GetLocale("SmexyMats");
local AceConfig = LibStub("AceConfigDialog-3.0");
local name = "SmexyMats(SM)"; 
local version = "v4.5.2"; 
local isTooltipDone = nil;
local storedLink = nil;
local LID = nil;

function SmexyMats:OnInitialize()
	SmexyMats:RegisterChatCommand("sm", "ChatCommand");
	SmexyMats.db = LibStub("AceDB-3.0"):New("SmexyMatsDB", SmexyMats.defaults, "profile");
	LibStub("AceConfig-3.0"):RegisterOptionsTable("SmexyMats", SmexyMats.options);
	AceConfig:AddToBlizOptions("SmexyMats", "SmexyMats(SM)");
	local tooltipMethodHooks = {
		SetCurrencyToken = { nil, Hook_SetCurrencyToken },
		SetQuestItem = { function(self,type,index) storedLink = GetQuestItemLink(type,index) end, nil },
		SetQuestLogItem = { function(self,type,index) storedLink = GetQuestLogItemLink(type,index) end, nil },
		SetRecipeResultItem = { function(self, recipeID) if recipeID then storedLink = C_TradeSkillUI.GetRecipeItemLink(recipeID) end end, nil },
		SetRecipeReagentItem = { function(self, recipeID, reagentIndex) if recipeID and reagentIndex then storedLink = C_TradeSkillUI.GetRecipeReagentItemLink(recipeID, reagentIndex) end end, nil },
	};
	for m, hooks in pairs(tooltipMethodHooks) do
		InstallHook(GameTooltip, m, hooks[1], hooks[2]);
	end;
	if (SmexyMats:CheckDB() == true) then
		SmexyMats:HookTooltips(); 
		print(SmexyMats.Colors.wowtoken .. name,SmexyMats.Colors.legendary .. version,L["|rLoad Complete!"]);
	else
		print(SmexyMats.Colors.wowtoken .. name,SmexyMats.Colors.legendary .. version,SmexyMats.Colors.DeathKnight .. L["Failed! |rMissing Data-Tables. Reinstall SemxyMats(SM) to correct this issue or report the error to https://mods.curse.com/addons/wow/270824-smexymats"]);
	end;
end;	

function GetItemProperties(item)
	local retObj = {};
	if not item then return; end;
	retObj.aa, retObj.bb, retObj.cc, retObj.dd, retObj.ee, retObj.ff, retObj.gg, retObj.hh, retObj.ii, retObj.jj, retObj.kk, retObj.ll, retObj.mm, retObj.nn, retObj.oo, retObj.pp = GetItemInfo(item);
	if retObj.bb then
		retObj.ID = GetIDFromLink(retObj.bb);
	end;
	return retObj;
end;

function SmexyMats:HookTooltips()
	GameTooltip:HookScript("OnShow", JustTheTip);
	GameTooltip:HookScript("OnTooltipCleared", function(self) isTooltipDone = nil; end);
	GameTooltip:HookScript("OnTooltipSetItem", SmexyMats.ModifyItemTooltip);
	ItemRefTooltip:HookScript("OnShow", JustTheTip);
	ItemRefTooltip:HookScript("OnTooltipCleared", function(self) isTooltipDone = nil; end);
	ItemRefTooltip:HookScript("OnTooltipSetItem", SmexyMats.ModifyItemTooltip);
end;

function SmexyMats:ChatCommand()
	InterfaceOptionsFrame_OpenToCategory("SmexyMats(SM)");
	InterfaceOptionsFrame_OpenToCategory("SmexyMats(SM)");
end;

function JustTheTip(tooltip, ...)
	tooltip:SetBackdropColor(0,0,0,1); 
	tooltip:Show();
end;

function GetIDFromLink(link)
	if link then
		local linktype, id = string.match(link, "|H([^:]+):(%d+)");
		if id then 
			return tonumber(id);
		else
			return 0;
		end;
	end;
end;

function InstallHook(tooltip, method, prehook, posthook)
	local orig = tooltip[method];
	local stub = function(...)
		if prehook then 
			prehook(...);
		end;
		local a,b,c,d,e,f,g,h,i,j,k = orig(...);
		if posthook then 
			posthook(...);
		end;
		return a,b,c,d,e,f,g,h,i,j,k;
	end;
	tooltip[method] = stub;
end;

function SmexyMats:CheckDB()
	if (not SmexyMats.Legion_ReagentIDs) then return false; end;
	if (not SmexyMats.WoD_ReagentIDs) then return false; end;
	if (not SmexyMats.Mists_ReagentIDs) then return false; end;
	if (not SmexyMats.Cataclysm_ReagentIDs) then return false; end;
	if (not SmexyMats.WotLK_ReagentIDs) then return false; end;
	if (not SmexyMats.TBC_ReagentIDs) then return false; end;
	if (not SmexyMats.Vanilla_ReagentIDs) then return false; end;
	if (not SmexyMats.ItemsFrom_Alchemy) then return false; end;
	if (not SmexyMats.ItemsFor_Alchemy) then return false; end;
	if (not SmexyMats.ItemsFrom_Archaeology) then return false; end;
	if (not SmexyMats.ItemsFor_Archaeology) then return false; end;
	if (not SmexyMats.ItemsFrom_Blacksmithing) then return false; end;
	if (not SmexyMats.ItemsFor_Blacksmithing) then return false; end;
	if (not SmexyMats.ItemsFrom_Cooking) then return false; end;
	if (not SmexyMats.ItemsFor_Cooking) then return false; end;
	if (not SmexyMats.ItemsFrom_Enchanting) then return false; end;
	if (not SmexyMats.ItemsFor_Enchanting) then return false; end;
	if (not SmexyMats.ItemsFrom_Engineering) then return false; end;
	if (not SmexyMats.ItemsFor_Engineering) then return false; end;
	if (not SmexyMats.ItemsFrom_FirstAid) then return false; end;
	if (not SmexyMats.ItemsFor_FirstAid) then return false; end;
	if (not SmexyMats.ItemsFrom_Fishing) then return false; end;
	if (not SmexyMats.ItemsFor_Fishing) then return false; end;
	if (not SmexyMats.ItemsFrom_Herbalism) then return false; end;
	if (not SmexyMats.ItemsFor_Herbalism) then return false; end;
	if (not SmexyMats.ItemsFrom_Inscription) then return false; end;
	if (not SmexyMats.ItemsFor_Inscription) then return false; end;
	if (not SmexyMats.ItemsFrom_Jewelcrafting) then return false; end;
	if (not SmexyMats.ItemsFor_Jewelcrafting) then return false; end;
	if (not SmexyMats.ItemsFrom_Leatherworking) then return false; end;
	if (not SmexyMats.ItemsFor_Leatherworking) then return false; end;
	if (not SmexyMats.ItemsFrom_Mining) then return false; end;
	if (not SmexyMats.ItemsFor_Mining) then return false; end;
	if (not SmexyMats.ItemsFrom_Skinning) then return false; end;
	if (not SmexyMats.ItemsFor_Skinning) then return false; end;
	if (not SmexyMats.ItemsFrom_Tailoring) then return false; end;
	if (not SmexyMats.ItemsFor_Tailoring) then return false; end;
	if (not SmexyMats.ItemsFrom_Vendor) then return false; end;
	if (not SmexyMats.ItemsFrom_WorldDrop) then return false; end;
	if (not SmexyMats.ItemsFrom_ZoneDrop) then return false; end;
	if (not SmexyMats.ItemsFrom_PvP) then return false; end;
	if (not SmexyMats.ItemsFrom_WoD_Missions) then return false; end;
	if (not SmexyMats.ItemsFrom_Legion_Missions) then return false; end;
	return true;
end;

function SmexyMats:GetExPack(obj)
	if (obj.aa == nil) or (obj.aa == '') or (string.len(obj.aa) < 1) then return -1, SmexyMats.ExPacks[-1].color; end;
	if (SmexyMats:IsFrom_Legion(obj.ID)) == true then return 6, SmexyMats.ExPacks[6].color; end;
	if (SmexyMats:IsFrom_WoD(obj.ID)) == true then return 5, SmexyMats.ExPacks[5].color; end;
	if (SmexyMats:IsFrom_Mists(obj.ID)) == true then return 4, SmexyMats.ExPacks[4].color; end;
	if (SmexyMats:IsFrom_Cataclysm(obj.ID)) == true then return 3, SmexyMats.ExPacks[3].color; end;
	if (SmexyMats:IsFrom_WotLK(obj.ID)) == true then return 2, SmexyMats.ExPacks[2].color; end;
	if (SmexyMats:IsFrom_TBC(obj.ID)) == true then return 1, SmexyMats.ExPacks[1].color; end;
	if (SmexyMats:IsFrom_Vanilla(obj.ID)) == true then return 0, SmexyMats.ExPacks[0].color; end;
	if (SmexyMats.db.profile.Equipment == true) and (SmexyMats.db.profile.Contents == true) then
		local X, Y = SmexyMats:Gear_ExpackID(obj);
		if (X ~= -1) then return X, Y; end;
	end;
	return -1, SmexyMats.ExPacks[-1].color;
end;

function ProcessTooltip(tt, obj)
	local ItemInfoCached = true;
	if not obj.ID then ItemInfoCached = false; end;
	if (obj.ID == 0) and (TradeSkillFrame ~= nil) and TradeSkillFrame:IsVisible() then
		if (GetMouseFocus():GetName()) == "TradeSkillSkillIcon" then
			obj.ID = tonumber(GetTradeSkillItemLink(TradeSkillFrame.selectedSkill):match("item:(%d+):")) or nil
		else
			for i = 1, 8 do
				if (GetMouseFocus():GetName()) == "TradeSkillReagent"..i then
					obj.ID = tonumber(GetTradeSkillReagentItemLink(TradeSkillFrame.selectedSkill, i):match("item:(%d+):")) or nil
					break;
				end;
			end;
		end;
	end;
	if (obj.ID == 0) then ItemInfoCached = false; end;
	if (not ItemInfoCached) then
		if (SmexyMats.db.profile.ErrorReporting == true) then
			if (_G["GameTooltipTextLeft1"]:GetText() ~= SmexyMats.LID) then
				print(SmexyMats.Colors.wowtoken .. L["[SM]:"] .. SmexyMats.Colors.DeathKnight .. L["Cannot fetch information for:"]);
				print(SmexyMats.Colors.wowtoken .. L["[SM]:"] .. SmexyMats.Colors.common ..  _G["GameTooltipTextLeft1"]:GetText());
				print(SmexyMats.Colors.wowtoken .. L["[SM]:"] .. SmexyMats.Colors.yellow .. L["An info request has been sent to the server for details on this item."]);
				print(SmexyMats.Colors.wowtoken .. L["[SM]:"] .. SmexyMats.Colors.uncommon .. L["Temporary Solution:"]);
				print(SmexyMats.Colors.wowtoken .. L["[SM]:"] .. SmexyMats.Colors.yellow .. L["1. Link the item in chat."]);
				print(SmexyMats.Colors.wowtoken .. L["[SM]:"] .. SmexyMats.Colors.yellow .. L["2. Click on it."]);
				print(SmexyMats.Colors.wowtoken .. L["[SM]:"] .. SmexyMats.Colors.uncommon .. L["To turn off these errors:"]);
				print(SmexyMats.Colors.wowtoken .. L["[SM]:"] .. SmexyMats.Colors.yellow .. L["type /sm and uncheck 'Report errors to chat'."]);
				print(SmexyMats.Colors.wowtoken .. L["[SM]:"] .. SmexyMats.Colors.uncommon .. L["More Info:"]);
				print(SmexyMats.Colors.wowtoken .. L["[SM]:"] .. SmexyMats.Colors.yellow .. L["More info on why this error occured can be found here:"]);
				print(SmexyMats.Colors.Mage .. "http://wowprogramming.com/docs/api/GetItemInfo");
				LID = _G["GameTooltipTextLeft1"]:GetText();
			end;
		end;
		return;
	end;
	
	local EP, color = SmexyMats:GetExPack(obj);
	local ProFor = SmexyMats:FormatToolTipString(obj.ID, 1); 
	local ProFrom = SmexyMats:FormatToolTipString(obj.ID, 2);
	
	tt:AddLine(" ",1,1,1);
	
	if (SmexyMats.db.profile.Contents == true) then 
		if EP ~= -1 then 
			local e = SmexyMats.ExPacks[EP].name; 
			tt:AddLine(SmexyMats.Colors.wowtoken .. L["[SM]Expansion: "] .. color .. e,1,1,1,false); 
		end; 
	end;
	if (SmexyMats.db.profile.Sources == true) then
		if (obj.aa) and (ProFrom) and (#(ProFrom) > 0) then 
			tt:AddLine(SmexyMats.Colors.wowtoken .. L["[SM]Source(s): "] .. SmexyMats.Colors.white .. ProFrom,1,1,1,true); 
		end;
	end;
	if (SmexyMats.db.profile.Professions == true) then
		if (obj.aa) and (ProFor) and (#(ProFor) > 0) then 
			tt:AddLine(SmexyMats.Colors.wowtoken .. L["[SM]Profession(s): "] .. SmexyMats.Colors.white .. ProFor,1,1,1,true); 
		end;
	end;
	if (SmexyMats.db.profile.ItemIDs == true) then 
		tt:AddLine(SmexyMats.Colors.wowtoken .. L["[SM]ItemID: "] .. SmexyMats.Colors.Paladin .. obj.ID,1,1,1,true); 
	end;
end;

function ExamineObject(obj)
	if not obj then return false; end;
	if (not obj.aa) or (obj.aa == "") or (not obj.bb) or (not obj.ID) or (obj.ID == 0) then return false; end;
	return true;
end;

function SmexyMats.ModifyItemTooltip(tt, ...)
	if (SmexyMats.db.profile.Enabled == false) then return; end;
	local obj, name, link, itemAquired = nil, nil, nil, false;
	local objTXT = _G["GameTooltipTextLeft1"]:GetText();
	if (not isTooltipDone) and tt then
		isTooltipDone = true;
		name, link = tt:GetItem();
		obj = GetItemProperties(link)
		if (ExamineObject(obj)) and (ExamineObject(obj)) then
			itemAquired = true;
		else
			link = storedLink;
			obj = GetItemProperties(link);
			if (ExamineObject(obj) == true) and (ExamineObject(obj) == true) then
				itemAquired = true;
			else
				obj = GetItemProperties(objTXT);
				if (ExamineObject(obj)) and (ExamineObject(obj)) then
					itemAquired = true;
				end;
			end;
		end;
		if itemAquired then
			ProcessTooltip(tt, obj);
		end;
	end;
end;

function SmexyMats:FormatToolTipString(iID, opt)
	local ttl = SmexyMats:SearchReagentDB(iID, opt); 
	local tts = "";
	if (#(ttl) == 0) then 
		return; 
	end; 
	table.sort(ttl);
	for k, v in pairs(ttl) do 
		if (tts == "") then 
			tts = SmexyMats:trim(v); 
		else 
			tts = tts .. ", " .. SmexyMats:trim(v); 
		end; 
	end; 
	return tts;
end;

function SmexyMats:SearchReagentDB(iID, opt)
	local x = {}; if not iID then return; end;
	if (opt == 2) then if (SmexyMats:IsFrom_Legion_Missions(iID)) == true then table.insert(x, SmexyMats.Profs[0.3]); end; end;
	if (opt == 2) then if (SmexyMats:IsFrom_WoD_Missions(iID)) == true then table.insert(x, SmexyMats.Profs[0.2]); end; end;
	if (opt == 2) then if (SmexyMats:IsFrom_PvP(iID)) == true then table.insert(x, SmexyMats.Profs[-2]); end; end;
	if (opt == 2) then if (SmexyMats:IsFrom_Vendor(iID)) == true then table.insert(x, SmexyMats.Profs[-1]); end; end;
	if (opt == 2) then if (SmexyMats:Is_WorldDrop(iID)) == true then table.insert(x, SmexyMats.Profs[0]); end; end;
	if (opt == 2) then if (SmexyMats:Is_ZoneDrop(iID)) == true then table.insert(x, SmexyMats.Profs[0.1]); end; end;
	if (opt == 1) then if (SmexyMats:IsFor_Alchemy(iID)) == true then table.insert(x, SmexyMats.Profs[1]); end;  end;
	if (opt == 2) then if (SmexyMats:IsFrom_Alchemy(iID)) == true then table.insert(x, SmexyMats.Profs[1]);end;  end;
	if (opt == 1) then if (SmexyMats:IsFor_Archaeology(iID)) == true then table.insert(x, SmexyMats.Profs[2]);end;  end;
	if (opt == 2) then if (SmexyMats:IsFrom_Archaeology(iID)) == true then table.insert(x, SmexyMats.Profs[2]);end;  end;
	if (opt == 1) then if (SmexyMats:IsFor_Blacksmithing(iID)) == true then table.insert(x, SmexyMats.Profs[3]);end;  end;
	if (opt == 2) then if (SmexyMats:IsFrom_Blacksmithing(iID)) == true then table.insert(x, SmexyMats.Profs[3]);end;  end;
	if (opt == 1) then if (SmexyMats:IsFor_Cooking(iID)) == true then table.insert(x, SmexyMats.Profs[4]);end;  end;
	if (opt == 2) then if (SmexyMats:IsFrom_Cooking(iID)) == true then table.insert(x, SmexyMats.Profs[4]);end;  end;
	if (opt == 1) then if (SmexyMats:IsFor_Enchanting(iID)) == true then table.insert(x, SmexyMats.Profs[5]);end;  end;
	if (opt == 2) then if (SmexyMats:IsFrom_Enchanting(iID)) == true then table.insert(x, SmexyMats.Profs[5]);end;  end;
	if (opt == 1) then if (SmexyMats:IsFor_Engineering(iID)) == true then table.insert(x, SmexyMats.Profs[6]);end;  end;
	if (opt == 2) then if (SmexyMats:IsFrom_Engineering(iID)) == true then table.insert(x,SmexyMats.Profs[6]);end;  end;
	if (opt == 1) then if (SmexyMats:IsFor_FirstAid(iID)) == true then table.insert(x, SmexyMats.Profs[7]);end;  end;
	if (opt == 2) then if (SmexyMats:IsFrom_FirstAid(iID)) == true then table.insert(x, SmexyMats.Profs[7]);end;  end;
	if (opt == 1) then if (SmexyMats:IsFor_Fishing(iID)) == true then table.insert(x, SmexyMats.Profs[8]);end;  end;
	if (opt == 2) then if (SmexyMats:IsFrom_Fishing(iID)) == true then table.insert(x, SmexyMats.Profs[8]);end;  end;
	if (opt == 2) then if (SmexyMats:IsFrom_Herbalism(iID)) == true then table.insert(x, SmexyMats.Profs[9]);end;  end;
	if (opt == 1) then if (SmexyMats:IsFor_Inscription(iID)) == true then table.insert(x, SmexyMats.Profs[10]);end;  end;
	if (opt == 2) then if (SmexyMats:IsFrom_Inscription(iID)) == true then table.insert(x, SmexyMats.Profs[10]);end;  end;
	if (opt == 1) then if (SmexyMats:IsFor_Jewelcrafting(iID)) == true then table.insert(x, SmexyMats.Profs[11]);end;  end;
	if (opt == 2) then if (SmexyMats:IsFrom_Jewelcrafting(iID)) == true then table.insert(x, SmexyMats.Profs[11]);end;  end;
	if (opt == 1) then if (SmexyMats:IsFor_Leatherworking(iID)) == true then table.insert(x, SmexyMats.Profs[12]);end;  end;
	if (opt == 2) then if (SmexyMats:IsFrom_Leatherworking(iID)) == true then table.insert(x, SmexyMats.Profs[12]);end;  end;
	if (opt == 1) then if (SmexyMats:IsFor_Mining(iID)) == true then table.insert(x, SmexyMats.Profs[13]);end;  end;
	if (opt == 2) then if (SmexyMats:IsFrom_Mining(iID)) == true then table.insert(x, SmexyMats.Profs[13]);end;  end;
	if (opt == 2) then if (SmexyMats:IsFrom_Skinning(iID)) == true then table.insert(x, SmexyMats.Profs[14]);end;  end;
	if (opt == 1) then if (SmexyMats:IsFor_Tailoring(iID)) == true then table.insert(x, SmexyMats.Profs[15]);end;  end;
	if (opt == 2) then if (SmexyMats:IsFrom_Tailoring(iID)) == true then table.insert(x, SmexyMats.Profs[15]);end;  end;
	return x;
end;